package de.rossmann.iledocs;

import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

/**
 * This class connects to the ILEDocs database and retrieves the documentation for the
 * passed program. 
 * 
 * A standalone HTML page is generated.
 * 
 * @author Mihael Schmidt
 *
 */
public class ILEDocs 
{
	public static SimpleDateFormat dateFormat = new SimpleDateFormat("'KW' w , EEEEE dd. MMMMM yyyy");
	
	private String library = null;
	private String file = null;
	private String member = null;
	
	/**
	 * @param args Three arguments are needed: Library, Source file and member
	 */
	public static void main(String[] args) 
	{
		if (args.length < 3)
		{
			System.out.println("Parameter: Library Sourcefile Member");
			System.exit(1);
		}

		ILEDocs main = new ILEDocs(args[0], args[1], args[2]);
		main.init();
		main.createDocumentation();
		main.dispose();
	}

	public ILEDocs(String library, String file, String member)
	{
		this.library = library;
		this.file = file;
		this.member = member;
	}
	
	public void init()
	{
		try
		{
			Properties databaseProperties = new Properties();
			databaseProperties.load(ILEDocs.class.getClassLoader().getResourceAsStream("database.properties"));
			DataAccess.init(databaseProperties);
			
			Properties velocityProperties = new Properties();
			velocityProperties.load(this.getClass().getResourceAsStream("/de/rossmann/iledocs/velocity.properties"));
			Velocity.init(velocityProperties);
		}
		catch(Exception e)
		{
			System.err.println("Error during initialization: " + e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
	}
	
	public void dispose()
	{
		DataAccess.close();
	}
	
	public void createDocumentation()
	{
		VelocityContext context = new VelocityContext();
		
		try
		{
			FileWriter w = new FileWriter(member.toLowerCase() + ".html");

			HashMap program = new HashMap();
			program.put("name", DataAccess.getProgramName(library, file, member));
			program.put("brief", DataAccess.getBriefProgramDesc(library, file, member));
			program.put("desc", DataAccess.getProgramDesc(library, file, member));
			program.put("date", DataAccess.getProgramDate(library, file, member));
			program.put("author", DataAccess.getProgramAuthor(library, file, member));
			program.put("todos", DataAccess.getProgramTodos(library, file, member));
			program.put("links", DataAccess.getProgramLinks(library, file, member));
			program.put("warnings", DataAccess.getProgramWarnings(library, file, member));
			program.put("revisions", DataAccess.getProgramRevisions(library,file,member));
			program.put("deprecated", DataAccess.getProgramDeprecated(library,file,member));
			program.put("infos", DataAccess.getProgramInfos(library, file, member));
			program.put("auths", DataAccess.getProgramAuths(library, file, member));
			program.put("configs", DataAccess.getProgramConfigs(library, file, member));
			program.put("sourceAttribute", DataAccess.getProgramSourceAttribute(library, file, member));

			context.put("creationDate", new Date().toString());
			context.put("program", program);
			context.put("files", DataAccess.getFiles(library, file, member));
			context.put("procedures", DataAccess.getProcedures(library, file, member));
			context.put("library", library);
			context.put("sourcefile", file);
			context.put("member", member);
			context.put("copybooks", DataAccess.getIncludes(library,file,member));
			context.put("critical", String.valueOf(DataAccess.isProgramCritical(library, file, member)));
			
			Velocity.mergeTemplate("/de/rossmann/iledocs/program.vm", context, w );
			w.flush();
			w.close();
		}
		catch(Exception e)
		{
			System.err.println("Could not create documentation: " + e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
	}
}
